package de.ozgcloud.eingang.common.formdata;

import static de.ozgcloud.eingang.common.formdata.IncomingFileTestFactory.*;
import static org.assertj.core.api.Assertions.*;

import java.io.File;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.function.Predicate;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import lombok.SneakyThrows;

class IncomingFileTest {

	private static final String TMP_DIRECTORY_PATH = System.getProperty("java.io.tmpdir");

	private static final Predicate<Path> hasNameSuffix = path -> path.getFileName().toString().endsWith(IncomingFile.TMP_FILE_SUFFIX);

	@Nested
	class TestTmpFile {

		@BeforeEach
		void init() {
			cleanupTempFiles();
		}

		@Test
		void shouldCreateTmpFile() {
			IncomingFileTestFactory.create().getContentStream();

			assertThat(noFilesWithSuffixInTempDirectory()).isFalse();
		}

		@Test
		void shouldCreateDeleteOnCloseInputStream() {
			var file = IncomingFileTestFactory.create();
			file.getContentStream();
			InputStream newStream = file.getContentStreamForFinalRead();

			assertThat(newStream).isInstanceOf(DeleteOnCloseInputStream.class);
		}

		@Test
		void validateInputStreamContent() {
			InputStream newStream = IncomingFileTestFactory.create().getContentStream();

			byte[] text = read(newStream);

			assertThat(text).isEqualTo(CONTENT);
		}

		@Test
		void shouldReturnNewInputStreamOnEveryRequest() {
			IncomingFile incomingFile = IncomingFileTestFactory.create();

			InputStream stream1 = incomingFile.getContentStream();
			InputStream stream2 = incomingFile.getContentStream();

			assertThat(stream1).isNotSameAs(stream2);
		}

		@Test
		void contentStreamShouldBeMultipleReadable() {
			IncomingFile incomingFile = IncomingFileTestFactory.create();

			InputStream stream1 = incomingFile.getContentStream();
			InputStream stream2 = incomingFile.getContentStream();

			assertThat(read(stream1)).isEqualTo(CONTENT);
			assertThat(read(stream2)).isEqualTo(CONTENT);

			InputStream stream3 = incomingFile.getContentStream();
			assertThat(read(stream3)).isEqualTo(CONTENT);
		}

		@SneakyThrows
		private byte[] read(InputStream stream) {
			return stream.readAllBytes();
		}
	}

	@SneakyThrows
	private static void cleanupTempFiles() {
		Files.walk(Path.of(TMP_DIRECTORY_PATH), 1).filter(hasNameSuffix).map(Path::toFile).forEach(File::delete);
	}

	@SneakyThrows
	private static boolean noFilesWithSuffixInTempDirectory() {
		return Files.walk(Path.of(TMP_DIRECTORY_PATH), 1).noneMatch(hasNameSuffix);
	}

}
