package de.ozgcloud.eingang.semantik.formbased.mantelantrag;

import java.util.Map;
import java.util.Optional;
import java.util.function.UnaryOperator;
import java.util.stream.IntStream;

import org.apache.commons.collections.MapUtils;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Component;

import de.ozgcloud.eingang.common.errorhandling.TechnicalException;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormHeader;
import de.ozgcloud.eingang.common.formdata.ZustaendigeStelle;
import de.ozgcloud.eingang.semantik.formbased.FormBasedMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Component
@RequiredArgsConstructor
@Log4j2
public class MantelantragZustaendigeStelleMapper implements FormBasedMapper {
	static final String MANTELANTRAG_FORM_ID = "maa_mantelantrag/maa_mantelantrag";
	static final String ZUSTELLUNG_NACHRICHTENBROKER_FIELD = "zustellung_nachrichtenbroker";
	static final String ORGANISATIONSEINHEIT_ID_FIELD = "kontaktsystem_oeid";
	static final String BEZEICHNUNG_FIELD = "OrganisationseinheitenBEZEICHNUNG";
	static final String EMAIL_FIELD = "zust_emailadresse";
	static final String HAUSANSCHRIFT_STRASSE_FIELD = "zust_strasse";
	static final String HAUSANSCHRIFT_PLZ_FIELD = "zust_postleitzahl";
	static final String HAUSANSCHRIFT_ORT_FIELD = "zust_ort";
	static final String TELEFON_FIELD = "telefon";

	private final Environment environment;

	@Override
	public boolean isResponsible(FormData formData) {
		return Optional.ofNullable(formData.getHeader())
				.map(FormHeader::getFormId)
				.map(v -> v.startsWith(MANTELANTRAG_FORM_ID))
				.orElse(false);
	}

	@Override
	public FormData parseFormData(FormData formData) {
		try {
			return adaptFormDataWithPossibleException(formData);
		} catch (TechnicalException exception) {
			LOG.warn("Mantelantrag is invalid: %s".formatted(exception.getMessage()));
			return formData;
		}
	}

	FormData adaptFormDataWithPossibleException(FormData formData) {
		var fieldMap = formData.getFormData();
		var slotIndex = findSlotIndex(fieldMap);
		return adaptFormDataWithValueGetter(formData, field -> getFieldByKeyOrEmpty(
				fieldMap,
				getNameForSlotIndex(field, slotIndex)
		));
	}

	private FormData adaptFormDataWithValueGetter(FormData formData, UnaryOperator<String> getValueOrEmpty) {
		return formData.toBuilder()
				.zustaendigeStelle(ZustaendigeStelle.builder()
						.organisationseinheitenId(getOrganisationseinheitId(getValueOrEmpty))
						.bezeichnung(getValueOrEmpty.apply(BEZEICHNUNG_FIELD))
						.email(getValueOrEmpty.apply(EMAIL_FIELD))
						.hausanschriftStrasse(getValueOrEmpty.apply(HAUSANSCHRIFT_STRASSE_FIELD))
						.hausanschriftPlz(getValueOrEmpty.apply(HAUSANSCHRIFT_PLZ_FIELD))
						.hausanschriftOrt(getValueOrEmpty.apply(HAUSANSCHRIFT_ORT_FIELD))
						.telefon(getValueOrEmpty.apply(TELEFON_FIELD))
						.build())
				.build();
	}

	private String getOrganisationseinheitId(UnaryOperator<String> getValueOrEmpty) {
		var organisationseinheitId = getValueOrEmpty.apply(ORGANISATIONSEINHEIT_ID_FIELD);

		if (organisationseinheitId.isBlank()) {
			throw new TechnicalException("OrganistaionseinheitId not found!");
		}

		return organisationseinheitId;
	}

	int findSlotIndex(Map<String, Object> fieldMap) {
		var identifier = environment.getProperty("ozgcloud.xta.identifier");
		return IntStream.range(0, 3)
				.filter(slotIndex -> getFieldByKeyOrEmpty(
								fieldMap,
								getNameForSlotIndex(ZUSTELLUNG_NACHRICHTENBROKER_FIELD, slotIndex)
						).equals(identifier)
				)
				.findFirst()
				.orElseThrow(() -> new TechnicalException("No matching nachrichtenbroker address '%s' found!".formatted(identifier)));
	}

	String getNameForSlotIndex(String name, int slotIndex) {
		return slotIndex > 0 ? "%s%d".formatted(name, slotIndex + 1) : name;
	}

	private String getFieldByKeyOrEmpty(Map<String, Object> fieldMap, String key) {
		return Optional.ofNullable(MapUtils.getString(fieldMap, key))
				.map(String::trim)
				.orElse("");
	}

}
